#import "AppDelegate.h"
#import "SystemSetting.h"
#import "DatabaseManeger.h"
#import "AsReaderP252BManager.h"
@interface AppDelegate ()
@end

@implementation AppDelegate{
    SystemSetting * systemSetting;
    AsReaderP252BManager * p252bManager;
}
- (BOOL)application:(UIApplication *)application didFinishLaunchingWithOptions:(NSDictionary *)launchOptions {
    self.insertOperation = [[NSOperationQueue alloc] init];
    p252bManager = [AsReaderP252BManager sharedAsReaderP252BManager];
    systemSetting = [SystemSetting sharedSystemSetting];
    if (![systemSetting getNotFirstUsed]) {
        [systemSetting setNotFirstUsed:YES];
        [systemSetting setBuzzer:AsReaderP252B3Buzzer_High];
        [systemSetting setInventoryTime:40];
        [systemSetting setIdleTime:0];
        [systemSetting setAutoOffTime:300];
        [systemSetting setQValue:4];
        [systemSetting setSleepTime:300];
        [systemSetting setLinkProfileValue:1];
        [systemSetting setDefaultLinkProfileValue:1];
        [systemSetting setOperationTime:0];
        [systemSetting setPowerGain:300];
        [systemSetting setInventorySession:AsReaderP252BSession_S0];
        [systemSetting setSessionFlag:AsReaderP252BSessionFlag_AB];
        [systemSetting setContinuousMode:YES];
        [systemSetting setRssiMode:NO];
        [systemSetting setDisPlayPC:YES];
        [systemSetting setLbtItemData:nil];
        [systemSetting setEpcMaskData:nil];
        [systemSetting setBuzzerTime:1];
        [systemSetting setVibratorTime:1];
        [systemSetting setSelectionEpcMaskData:nil];;
        [systemSetting setFrequencyAutomatic:YES];
        [systemSetting setBarcodeTimeout:4];
        [systemSetting setBaudRate:2];
    }
//    [self redirectConsoleLogToDocumentFolder:@"log.txt"];
//    [self removeLogFile];
//    [self redirectNSLogToDocumentFolder];
    return YES;
}

- (void)applicationWillResignActive:(UIApplication *)application {
}

- (void)applicationDidEnterBackground:(UIApplication *)application {
}

- (void)applicationWillEnterForeground:(UIApplication *)application {
}

- (void)applicationDidBecomeActive:(UIApplication *)application {
}

- (void)applicationWillTerminate:(UIApplication *)application {
}
#pragma mark - Blaster Method
- (void)initMask:(NSString *)maskValue{
    if (maskValue != nil) {
        [p252bManager.asReaderP252B  clearEpcMask];
        int length = (int)maskValue.length * 4;
        [p252bManager.asReaderP252B addEpcMask:16 length:length mask:maskValue];
    }
}

- (void)exitMask {
    [p252bManager.asReaderP252B  clearEpcMask];
    NSArray *maskArray = [[SystemSetting sharedSystemSetting] getSelectionEpcMaskData];
    for (AsReaderP252BSelectMaskEPCParam *mask in maskArray) {
        [p252bManager.asReaderP252B addEpcMask:mask];
    }
}

#pragma mark - Utility Methods

-(UIViewController *)getCurrentVC
{
    UIViewController *result = nil;
    UIWindow * window = [[UIApplication sharedApplication] keyWindow];
    if (window.windowLevel != UIWindowLevelNormal)
    {
        NSArray *windows = [[UIApplication sharedApplication] windows];
        for(UIWindow * tmpWin in windows)
        {
            if (tmpWin.windowLevel == UIWindowLevelNormal)
            {
                window = tmpWin;
                break;
            }
        }
    }
    UIView *frontView = [[window subviews] objectAtIndex:0];
    id nextResponder = [frontView nextResponder];
    
    if ([nextResponder isKindOfClass:[UIViewController class]])
        result = nextResponder;
    else
        result = window.rootViewController;
    return result;
}

+ (MBProgressHUD *)showProgressBarHUDWithTitle:(NSString *)title{
    [AppDelegate dismissGlobalHUD];
    UIViewController *result = nil;
    UIWindow * window = [[UIApplication sharedApplication] keyWindow];
    if (window.windowLevel != UIWindowLevelNormal)
    {
        NSArray *windows = [[UIApplication sharedApplication] windows];
        for(UIWindow * tmpWin in windows)
        {
            if (tmpWin.windowLevel == UIWindowLevelNormal)
            {
                window = tmpWin;
                break;
            }
        }
    }
    UIView *frontView = [[window subviews] objectAtIndex:0];
    id nextResponder = [frontView nextResponder];

    if ([nextResponder isKindOfClass:[UIViewController class]])
        result = nextResponder;
    else
        result = window.rootViewController;
    MBProgressHUD *hud = [MBProgressHUD showHUDAddedTo:[result view] animated:YES];
    hud.mode = MBProgressHUDModeDeterminate;
    hud.labelText = title;
    return hud;
}
+ (MBProgressHUD *)showGlobalProgressHUDWithTitle:(NSString *)title {
    UIViewController *result = nil;
    UIWindow * window = [[UIApplication sharedApplication] keyWindow];
    if (window.windowLevel != UIWindowLevelNormal)
    {
        NSArray *windows = [[UIApplication sharedApplication] windows];
        for(UIWindow * tmpWin in windows)
        {
            if (tmpWin.windowLevel == UIWindowLevelNormal)
            {
                window = tmpWin;
                break;
            }
        }
    }
    UIView *frontView = [[window subviews] objectAtIndex:0];
    id nextResponder = [frontView nextResponder];
    
    if ([nextResponder isKindOfClass:[UIViewController class]])
        result = nextResponder;
    else
        result = window.rootViewController;
    MBProgressHUD *hud = [MBProgressHUD showHUDAddedTo:[result view] animated:YES];
    hud.labelText = title;
    return hud;
}
+ (void)dismissGlobalHUD {
    UIViewController *result = nil;
    UIWindow * window = [[UIApplication sharedApplication] keyWindow];
    if (window.windowLevel != UIWindowLevelNormal)
    {
        NSArray *windows = [[UIApplication sharedApplication] windows];
        for(UIWindow * tmpWin in windows)
        {
            if (tmpWin.windowLevel == UIWindowLevelNormal)
            {
                window = tmpWin;
                break;
            }
        }
    }
    UIView *frontView = [[window subviews] objectAtIndex:0];
    id nextResponder = [frontView nextResponder];
    
    if ([nextResponder isKindOfClass:[UIViewController class]])
        result = nextResponder;
    else
        result = window.rootViewController;
    [MBProgressHUD hideHUDForView:[result view]  animated:YES];
}

- (void)redirectNSLogToDocumentFolder{
    NSArray *paths = NSSearchPathForDirectoriesInDomains(NSDocumentDirectory,NSUserDomainMask, YES);
    NSString *documentsDirectory = [paths objectAtIndex:0];
    NSString *fileName =[NSString stringWithFormat:@"%@.txt",@"writelog"];
    NSString *logFilePath = [documentsDirectory stringByAppendingPathComponent:fileName];
    freopen([logFilePath cStringUsingEncoding:NSASCIIStringEncoding],"a+",stderr);
}

- (void) removeLogFile{
    NSArray *paths = NSSearchPathForDirectoriesInDomains(NSDocumentDirectory,NSUserDomainMask, YES);
    NSString *documentsDirectory = [paths objectAtIndex:0];
    NSString *fileName =[NSString stringWithFormat:@"%@.txt",@"writelog"];
    NSString *logFilePath = [documentsDirectory stringByAppendingPathComponent:fileName];
    float logSize = 0;
    NSFileManager* manager = [NSFileManager defaultManager];
    if ([manager fileExistsAtPath:logFilePath]){
        logSize = [[manager attributesOfItemAtPath:logFilePath error:nil] fileSize]/(1024.0*1024);
    }
    if (logSize >= 1 ) {
        if ([[NSFileManager defaultManager] fileExistsAtPath:logFilePath]) {
            [[NSFileManager defaultManager] removeItemAtPath:logFilePath error:nil];
        }
    }
}
- (void) redirectConsoleLogToDocumentFolder:(NSString*)filename
{
    NSArray *paths
    = NSSearchPathForDirectoriesInDomains
    (NSDocumentDirectory,
     NSUserDomainMask, YES);
    
    NSString *documentsDirectory
    = [paths objectAtIndex:0];
    
    NSString *logPath
    = [documentsDirectory stringByAppendingPathComponent:filename];
    
    freopen([logPath cStringUsingEncoding:NSASCIIStringEncoding], "w",stderr);
}
@end
