//
//  CSVData.m
//  AsReaderLR
//
//  Created by mac on 2017/8/7.
//  Copyright © 2017年 asterisk. All rights reserved.
//

#import "CSVData.h"
#import "AsTagItem.h"
#import "AsBarcodeItem.h"

@implementation CSVData
+ (NSString *)filePath:(NSString *)fileName export:(ExportType)type{
    
    NSArray *paths = NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, YES);
    NSString *path = [paths objectAtIndex:0];
    NSString *filePath = @"";
    switch (type) {
        case rfidExportType:
        {
            filePath = [path stringByAppendingPathComponent:@"RFID"];
        }
            break;
        case barcodeExportType:
        {
            filePath = [path stringByAppendingPathComponent:@"Barcode"];
        }
            break;
        default:
            break;
    }
    
    NSFileManager *fileManager = [NSFileManager defaultManager];
    if (![fileManager fileExistsAtPath:filePath]) {
        [fileManager createDirectoryAtPath:filePath withIntermediateDirectories:YES attributes:nil error:nil];
    }
    filePath = [filePath stringByAppendingPathComponent:fileName];
    return filePath;
}

//+ (BOOL) isFileExist:(NSString *)fileName {
//    
//    NSFileManager *fileManager = [NSFileManager defaultManager];
//    BOOL result = [fileManager fileExistsAtPath:[self filePath:fileName]];
//    return result;
//}
//
//+ (void) deleteFileForFileName:(NSString *)fileName {
//    
//    NSFileManager *fileManager = [NSFileManager defaultManager];
//    [fileManager removeItemAtPath:[self filePath:fileName] error:nil];
//}

+ (void) createCSVWithFileName:(NSString *)fileName export:(ExportType)type{
    
    NSFileManager *fileManager = [NSFileManager defaultManager];
    if (![fileManager createFileAtPath:[self filePath:fileName export:type]  contents:nil attributes:nil]) {
    }
};

+ (void) exportRfidCSV:(NSString *)fileName withArray:(NSMutableArray *)array export:(ExportType)type isEpc:(BOOL)isEpc {
    NSOutputStream *output = [[NSOutputStream alloc] initToFileAtPath:[self filePath:fileName export:type] append:YES];
    [output open];
    
    if (![output hasSpaceAvailable]) {
        UIAlertView *alert = [[UIAlertView alloc]initWithTitle:nil message:@"" delegate:nil cancelButtonTitle:nil otherButtonTitles:@"OK", nil];
        [alert setMessage:@"Export Error"];
        [alert show];
        return;
    } else {
        NSString *header;
        if (isEpc) {
            header = @"pcepc hex,time,count\n";
        }else{
            header = @"epc hex,time,count\n";
        }
        
        
        const uint8_t *headerString = (const uint8_t *)[header cStringUsingEncoding:NSUTF8StringEncoding];
        NSInteger headerLength = [header lengthOfBytesUsingEncoding:NSUTF8StringEncoding];
        NSInteger result = [output write:headerString maxLength:headerLength];
        if (result <= 0) {
            UIAlertView *alert = [[UIAlertView alloc]initWithTitle:nil message:@"" delegate:nil cancelButtonTitle:nil otherButtonTitles:@"OK", nil];
            [alert setMessage:@"Export Error"];
            [alert show];
            return;
        }
        
        for (AsTagItem *item in array) {
            NSString *row;
            if (isEpc) {
                row = [NSString stringWithFormat:@"%@,%@,%d\n", item.tag, item.dateTime, item.count];
            }else{
                row = [NSString stringWithFormat:@"%@,%@,%d\n", [item.tag substringFromIndex:4], item.dateTime, item.count];
            }
            const uint8_t *rowString = (const uint8_t *)[row cStringUsingEncoding:NSUTF8StringEncoding];
            NSInteger rowLength = [row lengthOfBytesUsingEncoding:NSUTF8StringEncoding];
            result = [output write:rowString maxLength:rowLength];
            if (result <= 0) {
                UIAlertView *alert = [[UIAlertView alloc]initWithTitle:nil message:@"" delegate:nil cancelButtonTitle:nil otherButtonTitles:@"OK", nil];
                [alert setMessage:@"RFID File Export Error"];
                [alert show];
                return;
            }
        }
        
        UIAlertView *alert = [[UIAlertView alloc]initWithTitle:nil message:@"" delegate:nil cancelButtonTitle:nil otherButtonTitles:@"OK", nil];
        [alert setMessage:@"RFID File Export Success"];
        [alert show];
        
        [output close];
    }
}
+ (void) exportBarCodeCSV:(NSString *)fileName withArray:(NSMutableArray *)array export:(ExportType)type {
    NSOutputStream *output = [[NSOutputStream alloc] initToFileAtPath:[self filePath:fileName export:type] append:YES];
    [output open];
    
    if (![output hasSpaceAvailable]) {
        UIAlertView *alert = [[UIAlertView alloc]initWithTitle:nil message:@"" delegate:nil cancelButtonTitle:nil otherButtonTitles:@"OK", nil];
        [alert setMessage:@"Export Error"];
        [alert show];
        return;
    } else {
        NSString *header;
        
        header = @"barcode,time\n";
        
        const uint8_t *headerString = (const uint8_t *)[header cStringUsingEncoding:NSUTF8StringEncoding];
        NSInteger headerLength = [header lengthOfBytesUsingEncoding:NSUTF8StringEncoding];
        NSInteger result = [output write:headerString maxLength:headerLength];
        if (result <= 0) {
            UIAlertView *alert = [[UIAlertView alloc]initWithTitle:nil message:@"" delegate:nil cancelButtonTitle:nil otherButtonTitles:@"OK", nil];
            [alert setMessage:@"Export Error"];
            [alert show];
            return;
        }
        
        for (AsBarcodeItem *item in array) {
            NSString *row = [NSString stringWithFormat:@"%@,%@\n", item.barcode, item.dateTime];
            const uint8_t *rowString = (const uint8_t *)[row cStringUsingEncoding:NSUTF8StringEncoding];
            NSInteger rowLength = [row lengthOfBytesUsingEncoding:NSUTF8StringEncoding];
            result = [output write:rowString maxLength:rowLength];
            if (result <= 0) {
                UIAlertView *alert = [[UIAlertView alloc]initWithTitle:nil message:@"" delegate:nil cancelButtonTitle:nil otherButtonTitles:@"OK", nil];
                [alert setMessage:@"Barcode File Export Error"];
                [alert show];
                return;
            }
            
        }
        
        UIAlertView *alert = [[UIAlertView alloc]initWithTitle:nil message:@"" delegate:nil cancelButtonTitle:nil otherButtonTitles:@"OK", nil];
        [alert setMessage:@"Barcode File Export Success"];
        [alert show];
        
        [output close];
    }
}
@end
