//
//  AsReaderP252BManager.m
//  AsReaderLR
//
//  Created by iOS on 15.6.21.
//  Copyright © 2021 asterisk. All rights reserved.
//

#import "AsReaderP252BManager.h"
#import "ShowLogView.h"
#import "SystemSetting.h"
#import <ExternalAccessory/ExternalAccessory.h>
@interface AsReaderP252BManager ()<AsReaderP252BDeviceManagerDelegate,AsReaderP252BDelegate>

@end
@implementation AsReaderP252BManager{
    AsReaderP252BDeviceManager * bluetoothManager;
    ShowLogView *showLogView;
}
static AsReaderP252BManager * _sharedAsReaderP252BManager;
+ (AsReaderP252BManager *)sharedAsReaderP252BManager{
    if (!_sharedAsReaderP252BManager) {
        _sharedAsReaderP252BManager = [[AsReaderP252BManager alloc] init];
    }
    return _sharedAsReaderP252BManager;
}
- (instancetype)init
{
    self = [super init];
    if (self) {
        bluetoothManager = [AsReaderP252BDeviceManager shared];
        bluetoothManager.delegate = self;
        showLogView = [ShowLogView sharedShowLogView];
    }
    return self;
}
-(void)setDelegate:(id<AsReaderP252BManagerDelegate>)delegate{
    _delegate = delegate;
}
- (void)startDiscovered{
    [bluetoothManager startDiscovered];
}
- (void)stopDiscovered{
    [bluetoothManager stopDiscovered];
}
- (void)connectWithAsReader:(AsReaderP252B *)asraeder{
    _asReaderP252B = asraeder;
    [_asReaderP252B setDelegate:self];
    [_asReaderP252B connect];
}
- (void)disconnectWithAsReader:(AsReaderP252B *)asraeder{

    [asraeder connect];
}

#pragma mark ------MRXBluetoothManagerDelegate------
- (void)didDiscoverAsReaderP252BDeviceList:(NSArray<AsReaderP252B *> *)asReaderP252BList {
    if(_delegate && [_delegate respondsToSelector:@selector(didDiscoverAsReaderP252BDeviceList:)]){
        [_delegate didDiscoverAsReaderP252BDeviceList:asReaderP252BList];
    }
    if ([[SystemSetting sharedSystemSetting] getIsAutoConnected] && (_asReaderP252B == nil || _asReaderP252B.isConnected == NO)) {
        for (AsReaderP252B *asReaderP252B in asReaderP252BList) {
            NSString * serialNumber = @"";
            if ([asReaderP252B.currentAccessory isKindOfClass:[EAAccessory class]]) {
                EAAccessory * accessory = asReaderP252B.currentAccessory;
                serialNumber = accessory.serialNumber;
            }
            if ([serialNumber isEqualToString:[[SystemSetting sharedSystemSetting] getAsReaderP252BName]]) {
                [self stopDiscovered];
                [self connectWithAsReader:asReaderP252B];
                return;
            }
        }
    }
}
- (void)didUpdateBluetoothState:(BOOL)stasus{
    if(_delegate && [_delegate respondsToSelector:@selector(didUpdateBluetoothState:)]){
        [_delegate didUpdateBluetoothState:stasus];
    }
}
#pragma mark ------AsReaderDelegate------
- (void)whenAsReaderP252BIsConnected:(BOOL)isConnected{
    if (isConnected) {
        _settingRFIDType = AsReaderP252BSettingRFIDType_Set;
    }else{
        _settingRFIDType = AsReaderP252BSettingRFIDType_UnSet;
    }
    [showLogView showView:isConnected];
    if(_delegate && [_delegate respondsToSelector:@selector(whenAsReaderP252BIsConnected:)]){
        [_delegate whenAsReaderP252BIsConnected:isConnected];
    }
}
- (void)updateAsReaderP252BDeviceState:(AsReaderP252BResultCode)error{
    if(_delegate && [_delegate respondsToSelector:@selector(updateAsReaderP252BDeviceState:)]){
        [_delegate updateAsReaderP252BDeviceState:error];
    }
}
- (void)readAsReaderP252BTag:(NSString *)tag rssi:(float)rssi phase:(float)phase frequency:(float)frequency{
    if(_delegate && [_delegate respondsToSelector:@selector(readAsReaderP252BTag:rssi:phase:frequency:)]){
        [_delegate readAsReaderP252BTag:tag rssi:rssi phase:phase frequency:frequency];
    }
}
- (void)changedAsReaderP252BActionState:(AsReaderP252BCommandType)action resultCode:(NSInteger)resultCode{
    if(_delegate && [_delegate respondsToSelector:@selector(changedAsReaderP252BActionState:resultCode:)]){
        [_delegate changedAsReaderP252BActionState:action resultCode:resultCode];
    }
}

- (void)accessAsReaderP252BResult:(AsReaderP252BResultCode)error actionState:(AsReaderP252BCommandType)action epc:(NSString *)epc data:(NSString *)data rssi:(float)rssi phase:(float)phase frequency:(float)frequency{
    if(_delegate && [_delegate respondsToSelector:@selector(accessAsReaderP252BResult:actionState:epc:data:rssi:phase:frequency:)]){
        [_delegate accessAsReaderP252BResult:error actionState:action epc:epc data:data rssi:rssi phase:phase frequency:frequency];
    }
}
- (void)readAsReaderP252BTemperatureTag:(NSString *)tag rssi:(float)rssi phase:(float)phase frequency:(float)frequency temperature:(float)temperature{
    if(_delegate && [_delegate respondsToSelector:@selector(readAsReaderP252BTemperatureTag:rssi:phase:frequency:temperature:)]){
        [_delegate readAsReaderP252BTemperatureTag:tag rssi:rssi phase:phase frequency:frequency temperature:temperature];
    }
}
- (void)readAsReaderP252BHumidityTag:(NSString *)tag rssi:(float)rssi phase:(float)phase frequency:(float)frequency humidity:(float)humidity{
    if(_delegate && [_delegate respondsToSelector:@selector(readAsReaderP252BHumidityTag:rssi:phase:frequency:humidity:)]){
        [_delegate readAsReaderP252BHumidityTag:tag rssi:rssi phase:phase frequency:frequency humidity:humidity];
    }
}

- (BOOL)onAsReaderP252BTriggerKeyEvent:(BOOL)status{
    if(_delegate && [_delegate respondsToSelector:@selector(onAsReaderP252BTriggerKeyEvent:)]){
        return [_delegate onAsReaderP252BTriggerKeyEvent:status];
    }
    return NO;
}
- (void)whenAsReaderP252BReceivedData:(NSData *)receivedData{
    [showLogView addData:receivedData isInput:YES];
    if(_delegate && [_delegate respondsToSelector:@selector(whenAsReaderP252BReceivedData:)]){
        [_delegate whenAsReaderP252BReceivedData:receivedData];
    }
}
- (void)whenAsReaderP252BWriteData:(NSData *)writeData{
    [showLogView addData:writeData isInput:NO];
    if(_delegate && [_delegate respondsToSelector:@selector(whenAsReaderP252BWriteData:)]){
        [_delegate whenAsReaderP252BWriteData:writeData];
    }
}
- (BOOL)onAsReaderP252BModeKeyEvent:(BOOL)status{
    if(_delegate && [_delegate respondsToSelector:@selector(onAsReaderP252BModeKeyEvent:)]){
        return [_delegate onAsReaderP252BModeKeyEvent:status];
    }
    return NO;
}
- (void)detectBarcode:(AsReaderP252BBarcodeType)barcodeType barcode:(NSString *)barcode{
    if(_delegate && [_delegate respondsToSelector:@selector(detectBarcode:barcode:)]){
        [_delegate detectBarcode:barcodeType barcode:barcode];
    }
}
- (void)detectBarcode:(AsReaderP252BBarcodeType)barcodeType barcodeData:(NSData *)barcodeData{
    if(_delegate && [_delegate respondsToSelector:@selector(detectBarcode:barcodeData:)]){
        [_delegate detectBarcode:barcodeType barcodeData:barcodeData];
    }
}

@end
