//
//  SystemSetting.m
//  AsReaderLR
//
//  Created by mac on 2017/8/7.
//  Copyright © 2017年 asterisk. All rights reserved.
//
#import "SystemSetting.h"
#import "AppDelegate.h"
#import "AsReaderP252BManager.h"

#define ISNOTFIRSTUSED          @"IsNotFirtUsed"
#define BUZZER                  @"Buzzer"
#define BUZZERTIME              @"BuzzerTime"
#define VIBRATORTIME            @"VibratorTime"
#define INVENTORYTIME           @"InventoryTime"
#define IDLETIME                @"IdleTime"
#define AUTOOFFTIME             @"AutoOffTime"
#define QVALUE                  @"QValue"
#define SLEEPTIME               @"SleepTime"
#define LINKPROFILEVALUE        @"LinkProfileValue"
#define DeFAULTLINKPROFILEVALUE @"DefaultLinkProfileValue"
#define OPERATIONTIME           @"OperationTime"
#define POWERGAIN               @"PowerGain"
#define INVENTORYSESSION        @"InventorySession"
#define SESSIONFLAG             @"SessionFlag"
#define CONTINUOUSMODE          @"ContinuousMode"
#define RSSIMODE                @"RssiMode"
#define DISPLAYPC               @"Displaypc"
#define LBTITEMDATA             @"LbtItemData"
#define RFIDDATA                @"RfidData"
#define MAPTAGS                 @"MapTags"
#define EPCMASKDATA             @"EpcMaskData"
#define SELECTIONEPCMASKDATA    @"SelectionEpcMaskData"
#define FREQUENCYAUTOMATIC      @"FrequencyAutomatic"
#define BATTERYINTERVAL         @"BatteryInterval"
#define BARCODERESCANTIME       @"BarcodeRescanTime"
#define BARCODEDATA             @"BarcodeData"
#define BARCODETIMEOUT          @"BarcodeTimeout"
#define BAUDRATE                @"BaudRate"

#define SETTING_KEY_IS_AUTO_CONNECTED  @"is_auto_connected"
#define SETTING_KEY_ASREADERP252B_NAME @"AsReaderP252BName_name"

@implementation SystemSetting{
    AppDelegate    * m_appdelegate;
    NSUserDefaults * m_userDefaults;
    AsReaderP252BManager      * m_p252bManager;
}

static SystemSetting * systemSetting;

+ (SystemSetting *)sharedSystemSetting{
    if (!systemSetting) {
        systemSetting = [[SystemSetting alloc] init];
    }
    return systemSetting;
}

- (instancetype)init
{
    self = [super init];
    if (self) {
        m_appdelegate = (AppDelegate *)[[UIApplication sharedApplication] delegate];
        m_userDefaults = [NSUserDefaults standardUserDefaults];
        m_p252bManager = [AsReaderP252BManager sharedAsReaderP252BManager];
    }
    return self;
}

- (void)RFIDSetting{
    if (m_p252bManager.asReaderP252B.isConnected) {
        m_p252bManager.asReaderP252B.buzzer = [self getBuzzer];
        m_p252bManager.asReaderP252B.sleepTime = [self getSleepTime];
        m_p252bManager.asReaderP252B.idleTime = [self getIdleTime];
        m_p252bManager.asReaderP252B.autoOffTime = [self getAutoOffTime];
        m_p252bManager.asReaderP252B.qValue = [self getQValue];
        m_p252bManager.asReaderP252B.linkProfileValue = [self getDefaultLinkProfileValue];
        m_p252bManager.asReaderP252B.operationTime = [self getOperationTime];
        m_p252bManager.asReaderP252B.powerGain = [self getPowerGain];
        m_p252bManager.asReaderP252B.inventorySession = [self getInventorySession];
        m_p252bManager.asReaderP252B.sessionFlag = [self getSessionFlag];
        m_p252bManager.asReaderP252B.continuousMode = [self getContinuousMode];
        m_p252bManager.asReaderP252B.rssiMode = [self getRssiMode];
        [m_p252bManager.asReaderP252B setBarcodeTimeOut:[self getBarcodeTimeout]];
        BOOL isJapan = [[m_p252bManager.asReaderP252B regionName] hasPrefix:@"Japan"];

        if (isJapan) {
            [m_p252bManager.asReaderP252B setFrequencyAutomatic:[self getFrequencyAutomatic]];
            if(![self getFrequencyAutomatic]){
                NSArray * lbtItemData = [self getLbtItemData];
                [m_p252bManager.asReaderP252B setFrequency:lbtItemData];
            }
        }
        [m_p252bManager.asReaderP252B clearEpcMask];
        for (AsReaderP252BSelectMaskEPCParam *mask in [self getSelectionEpcMaskData]) {
            [m_p252bManager.asReaderP252B addEpcMask:mask];
        }
        [m_p252bManager.asReaderP252B setBaudRate:[self getBaudRate]];
    }
}

- (void)setNotFirstUsed:(BOOL)isFirst{
    [m_userDefaults setBool:isFirst forKey:ISNOTFIRSTUSED];
    [m_userDefaults synchronize];
}
- (BOOL)getNotFirstUsed{
    return [m_userDefaults boolForKey:ISNOTFIRSTUSED];
}


- (void)setBuzzer:(int)buzzrtType{
    [m_userDefaults setInteger:buzzrtType forKey:BUZZER];
    [m_userDefaults synchronize];
}
 
- (void)setInventoryTime:(int)inventoryTime{
    [m_userDefaults setInteger:inventoryTime forKey:INVENTORYTIME];
    [m_userDefaults synchronize];
}
- (void)setIdleTime:(int)idleTime{
    [m_userDefaults setInteger:idleTime forKey:IDLETIME];
    [m_userDefaults synchronize];
}
- (void)setAutoOffTime:(int)autoOffTime{
    [m_userDefaults setInteger:autoOffTime forKey:AUTOOFFTIME];
    [m_userDefaults synchronize];
}
- (void)setQValue:(int)qValue{
    [m_userDefaults setInteger:qValue forKey:QVALUE];
    [m_userDefaults synchronize];
}
- (void)setSleepTime:(int)sleepTimeValue{
    [m_userDefaults setInteger:sleepTimeValue forKey:SLEEPTIME];
    [m_userDefaults synchronize];
}
- (void)setLinkProfileValue:(int)linkProfileValue{
    [m_userDefaults setInteger:linkProfileValue forKey:LINKPROFILEVALUE];
    [m_userDefaults synchronize];
}
- (void)setDefaultLinkProfileValue:(int)defaultLinkProfileValue{
    [m_userDefaults setInteger:defaultLinkProfileValue forKey:DeFAULTLINKPROFILEVALUE];
    [m_userDefaults synchronize];
}
- (void)setOperationTime:(int)operationTime{
    [m_userDefaults setInteger:operationTime forKey:OPERATIONTIME];
    [m_userDefaults synchronize];
}
- (void)setPowerGain:(int)powerGain{
    [m_userDefaults setInteger:powerGain forKey:POWERGAIN];
    [m_userDefaults synchronize];
}
- (void)setInventorySession:(int)inventorySession{
    [m_userDefaults setInteger:inventorySession forKey:INVENTORYSESSION];
    [m_userDefaults synchronize];
}
- (void)setSessionFlag:(int)sessionFlag{
    [m_userDefaults setInteger:sessionFlag forKey:SESSIONFLAG];
    [m_userDefaults synchronize];
}

- (void)setContinuousMode:(BOOL)isContinuousMode{
    [m_userDefaults setInteger:isContinuousMode forKey:CONTINUOUSMODE];
    [m_userDefaults synchronize];
}
- (void)setRssiMode:(BOOL)isRssiMode{
    [m_userDefaults setInteger:isRssiMode forKey:RSSIMODE];
    [m_userDefaults synchronize];
}
- (void)setDisPlayPC:(BOOL)isDisplaypc{
    [m_userDefaults setBool:isDisplaypc forKey:DISPLAYPC];
    [m_userDefaults synchronize];
}

- (void)setLbtItemData:(NSArray *)lbtItemData{
    NSData *lbtData = [NSKeyedArchiver archivedDataWithRootObject:lbtItemData];
    [m_userDefaults setObject:lbtData forKey:LBTITEMDATA];
    lbtData = nil;
    [m_userDefaults synchronize];
}
- (void)setRfidData:(NSArray *)rfidData{
    NSData *data = [NSKeyedArchiver archivedDataWithRootObject:rfidData];
    [m_userDefaults setObject:data forKey:RFIDDATA];
     data = nil;
    [m_userDefaults synchronize];
}
- (void)setMapTags:(NSDictionary *)mapTags{
    NSData *data = [NSKeyedArchiver archivedDataWithRootObject:mapTags];
    [m_userDefaults setObject:data forKey:MAPTAGS];
     data = nil;
    [m_userDefaults synchronize];
}
- (void)setEpcMaskData:(NSArray *)epcMaskData{
    NSData *epcData = [NSKeyedArchiver archivedDataWithRootObject:epcMaskData];
    [m_userDefaults setObject:epcData forKey:EPCMASKDATA];
    epcData = nil;
    [m_userDefaults synchronize];
} 

- (int)getBuzzer{
    return (int)[m_userDefaults integerForKey:BUZZER];
}
- (int)getInventoryTime{
    return (int)[m_userDefaults integerForKey:INVENTORYTIME];
}
- (int)getIdleTime{
    return (int)[m_userDefaults integerForKey:IDLETIME];
}
- (int)getAutoOffTime{
    return (int)[m_userDefaults integerForKey:AUTOOFFTIME];
}
- (int)getQValue{
    return (int)[m_userDefaults integerForKey:QVALUE];
}
- (int)getSleepTime{
    return (int)[m_userDefaults integerForKey:SLEEPTIME];
}
- (int)getLinkProfileValue{
    return (int)[m_userDefaults integerForKey:LINKPROFILEVALUE];
}
- (int)getDefaultLinkProfileValue{
    return (int)[m_userDefaults integerForKey:DeFAULTLINKPROFILEVALUE];
}
- (int)getOperationTime{
    return (int)[m_userDefaults integerForKey:OPERATIONTIME];
}
- (int)getPowerGain{
    return (int)[m_userDefaults integerForKey:POWERGAIN];
}
- (int)getInventorySession{
    return (int)[m_userDefaults integerForKey:INVENTORYSESSION];
}
- (int)getSessionFlag{
    return (int)[m_userDefaults integerForKey:SESSIONFLAG];
}
- (BOOL)getContinuousMode{
    return [m_userDefaults boolForKey:CONTINUOUSMODE];
}
- (BOOL)getRssiMode{
    return [m_userDefaults boolForKey:RSSIMODE];
}
- (BOOL)getDisPlayPC{
    return [m_userDefaults boolForKey:DISPLAYPC];
}


- (NSArray *)getLbtItemData{
    NSData *tokenObject = [m_userDefaults objectForKey:LBTITEMDATA];
    NSArray * array = [NSKeyedUnarchiver unarchiveObjectWithData:tokenObject];
    return array;
}
- (NSArray *)getRfidData{
    NSData *data = [m_userDefaults objectForKey:RFIDDATA];
    NSArray * array = [NSKeyedUnarchiver unarchiveObjectWithData:data];
    return array;
}
- (NSDictionary *)getMapTags{
    NSData *data = [m_userDefaults objectForKey:MAPTAGS];
    NSDictionary * dic = [NSKeyedUnarchiver unarchiveObjectWithData:data];
    return dic;
}
- (NSArray *)getEpcMaskData{
    NSData *tokenObject = [m_userDefaults objectForKey:EPCMASKDATA];
    NSArray * array = [NSKeyedUnarchiver unarchiveObjectWithData:tokenObject];
    return array;
}
- (void)setBuzzerTime:(int)buzzerTime{
    [m_userDefaults setInteger:buzzerTime forKey:BUZZERTIME];
    [m_userDefaults synchronize];
}
- (int)getBuzzerTime{
    return (int)[m_userDefaults integerForKey:BUZZERTIME];
}

- (void)setVibratorTime:(int)vibratorTime{
    [m_userDefaults setInteger:vibratorTime forKey:VIBRATORTIME];
    [m_userDefaults synchronize];
}
- (int)getVibratorTime{
    return (int)[m_userDefaults integerForKey:VIBRATORTIME];
}
- (void)setFrequencyAutomatic:(BOOL)isAutomatic{
    [m_userDefaults setBool:isAutomatic forKey:FREQUENCYAUTOMATIC];
    [m_userDefaults synchronize];
}
- (BOOL)getFrequencyAutomatic{
    return [m_userDefaults boolForKey:FREQUENCYAUTOMATIC];
}
- (void)setSelectionEpcMaskData:(NSArray *)selectionEpcMaskData{
    NSData *selectionEpcData = [NSKeyedArchiver archivedDataWithRootObject:selectionEpcMaskData];
    [m_userDefaults setObject:selectionEpcData forKey:SELECTIONEPCMASKDATA];
    selectionEpcData = nil;
    [m_userDefaults synchronize];
}
- (NSArray *)getSelectionEpcMaskData{
    NSData *tokenObject = [m_userDefaults objectForKey:SELECTIONEPCMASKDATA];
    NSArray * array = [NSKeyedUnarchiver unarchiveObjectWithData:tokenObject];
    return array;
}
- (void)setBatteryInterval:(NSString *)batteryInterval {
    NSUserDefaults *prefs = [NSUserDefaults standardUserDefaults];
    [prefs setObject:batteryInterval forKey:BATTERYINTERVAL];
    [prefs synchronize];
}

- (NSString *)getBatteryInterval {
    NSUserDefaults *prefs = [NSUserDefaults standardUserDefaults];
    NSString *batteryInterval = [prefs stringForKey:BATTERYINTERVAL];
    
    if (batteryInterval == nil || [batteryInterval isEqualToString:@""]) {
        batteryInterval = @"10";
    }
    return batteryInterval;
}

- (void)setBarcodeReScanTime:(int)barcodeReScanTime{
    [m_userDefaults setInteger:barcodeReScanTime forKey:BARCODERESCANTIME];
    [m_userDefaults synchronize];
}
- (int)getBarcodeReScanTime{
    return (int)[m_userDefaults integerForKey:BARCODERESCANTIME];
}

- (void)setBarcodeData:(NSArray *)barcodeData{
    NSData *data = [NSKeyedArchiver archivedDataWithRootObject:barcodeData];
    [m_userDefaults setObject:data forKey:BARCODEDATA];
    data = nil;
    [m_userDefaults synchronize];
}
- (NSArray *)getBarcodeData{
    NSData *data = [m_userDefaults objectForKey:BARCODEDATA];
    NSArray * array = [NSKeyedUnarchiver unarchiveObjectWithData:data];
    return array;
}

- (void)setBarcodeTimeout:(int)timeout{
    [m_userDefaults setInteger:timeout forKey:BARCODETIMEOUT];
    [m_userDefaults synchronize];
}
- (int)getBarcodeTimeout{
    return (int)[m_userDefaults integerForKey:BARCODETIMEOUT];
}

- (void)setBaudRate:(int)baudRate{
    [m_userDefaults setInteger:baudRate forKey:BAUDRATE];
    [m_userDefaults synchronize];
}
- (int)getBaudRate{
    return (int)[m_userDefaults integerForKey:BAUDRATE];
}

- (BOOL)getIsAutoConnected {
    return [m_userDefaults boolForKey:SETTING_KEY_IS_AUTO_CONNECTED];
}

- (void)setIsAutoConnected:(BOOL)isAutoConnected {
    [m_userDefaults setBool:isAutoConnected forKey:SETTING_KEY_IS_AUTO_CONNECTED];
    [m_userDefaults synchronize];
}

- (NSString *)getAsReaderP252BName {
    return [m_userDefaults stringForKey:SETTING_KEY_ASREADERP252B_NAME];
}
- (void)setAsReaderP252BName:(NSString *)AsReaderP252BName {
    [m_userDefaults setValue:AsReaderP252BName forKey:SETTING_KEY_ASREADERP252B_NAME];
    [m_userDefaults synchronize];
}
@end
