#import "InventoryViewController.h"
#import "InventoryTableViewCell.h"
#import "ReadMemoryViewController.h"
#import "WriteMemoryViewController.h"
#import "TagAccessViewController.h"
#import "SystemSetting.h"
#import "CSVData.h"
#import "DatabaseManeger.h"
#import "MBProgressHUD.h"
#import "CSVExportTableViewController.h"
#import "Define.h"
#import "ExcelData.h"
#import "AsReaderP252BManager.h"
#define ALERT_VIEW_OPERATION_TIME   1

#define PICKER_VIEW_INVENTORY_SESSION   0
#define PICKER_VIEW_POWER_GAIN          1
#define PICKER_VIEW_SESSION_FLAG        2
#define csvInvenShow @"csvInventoryShow"
#define TAG_PC_LENGTH               4
@interface InventoryViewController ()<AsReaderP252BManagerDelegate> {
    long currentTagCount;
    BOOL isCanScan;
    BOOL isJapan;
    SystemSetting * systemSetting;
    UIAlertView * currentAlertView;
    DatabaseManeger * databaseManeger;
    NSMutableDictionary * sqlDic;
    AsReaderP252BManager *p252bManager;
    NSMutableArray *tagArray;
}

@property (strong, nonatomic) IBOutlet UITableView *tableView;
@property (weak, nonatomic) IBOutlet UIPickerView *pickerView;
@property (weak, nonatomic) IBOutlet UIButton *inventoryBtn;
@property (weak, nonatomic) IBOutlet UIBarButtonItem *csvOutputBtn;
@property (weak, nonatomic) IBOutlet UILabel *speedLabel;
@property (weak, nonatomic) IBOutlet UILabel *tagCountLabel;
@property (weak, nonatomic) IBOutlet UILabel *totalTagCountLabel;
@property (weak, nonatomic) IBOutlet UISwitch *displayPcSwitch;
@property (weak, nonatomic) IBOutlet UISwitch *continuousModeSwitch;
@property (weak, nonatomic) IBOutlet UISwitch *reportRssiSwitch;
@property (weak, nonatomic) IBOutlet UIView *powerGainPickerTopView;
@property (weak, nonatomic) IBOutlet UITextField *powerGainTextField;
@property (weak, nonatomic) IBOutlet UIScrollView *scrollView;
@property (weak, nonatomic) IBOutlet UITextField *operationTimeTextField;
@property (strong, nonatomic) IBOutletCollection(UIControl) NSArray *toDisableElems;
@property (strong, nonatomic) NSArray *powerGainPickerData;
@property (weak, nonatomic) IBOutlet UIButton *OptionButton;
@property (weak, nonatomic) IBOutlet UITextField *inventorySessionTextField;

@property (readwrite, nonatomic) AsReaderP252BSessionType inventorySession;
@property (strong, nonatomic) NSArray *inventorySessionPickerData;
@property (weak, nonatomic) IBOutlet UITextField *sessionFlagTextField;
@property (readwrite, nonatomic) AsReaderP252BSessionFlag sessionFlag;
@property (strong, nonatomic) NSArray *sessionFlagPickerData;
@property (strong, nonatomic) UIAlertController *actionAlertVC;
@property (assign, nonatomic)NSInteger csvType;
@end

@implementation InventoryViewController

- (void)viewDidLoad {
    [super viewDidLoad];
    systemSetting = [SystemSetting sharedSystemSetting];
    p252bManager = [AsReaderP252BManager sharedAsReaderP252BManager];
    p252bManager.delegate = self;
    appDelegate = (AppDelegate *)[[UIApplication sharedApplication] delegate];
    
    appDelegate.insertOperation.maxConcurrentOperationCount=1;

    sqlDic = [[NSMutableDictionary alloc] init];
    
    databaseManeger = [DatabaseManeger sharedDatabaseManeger];
    [[databaseManeger getDatebaseQueue] inDatabase:^(FMDatabase *db) {
        aryTags = [[NSMutableArray alloc] initWithArray:[databaseManeger getAllAsTagItemsfmdb:db]];
    }];

    if (!aryTags) {
        [aryTags removeAllObjects];
        aryTags = [[NSMutableArray alloc] init];
    }else{
        mapTags = [[NSMutableDictionary alloc] init];
        for (AsTagItem * item in  aryTags) {
            [mapTags setObject:item forKey:item.tag];
        }
    }
    
    tagArray = [[NSMutableArray alloc] init];
}

- (UIAlertController *)actionAlertVC {
    if (!_actionAlertVC) {
        _actionAlertVC = [UIAlertController alertControllerWithTitle:nil message:nil preferredStyle:UIAlertControllerStyleActionSheet];
        
        UIAlertAction *csvAction = [UIAlertAction actionWithTitle:@"CSV Export" style:UIAlertActionStyleDefault handler:^(UIAlertAction * _Nonnull action) {
            [self csvExport];
        }];
        UIAlertAction *excelAction = [UIAlertAction actionWithTitle:@"Excel Export" style:UIAlertActionStyleDefault handler:^(UIAlertAction * _Nonnull action) {
            [self excelExport];
        }];
        UIAlertAction *excelListAction = [UIAlertAction actionWithTitle:@"Excel List" style:UIAlertActionStyleDefault handler:^(UIAlertAction * _Nonnull action) {
            self->_csvType = 2;
            [self performSegueWithIdentifier:csvInvenShow sender:nil];
        }];
        UIAlertAction *cancelAction = [UIAlertAction actionWithTitle:@"Cancel" style:UIAlertActionStyleCancel handler:nil];

        [_actionAlertVC addAction:csvAction];
        [_actionAlertVC addAction:excelAction];
        [_actionAlertVC addAction:excelListAction];
        [_actionAlertVC addAction:cancelAction];
    }
    return _actionAlertVC;
}

- (void)updateUI {
    [self.tagCountLabel setText:[NSString stringWithFormat:@"%d", (int)aryTags.count]];
    [self.totalTagCountLabel setText:[NSString stringWithFormat:@"%ld", mTotalTagCounts]];
    [self.speedLabel setText:[NSString stringWithFormat:@"%0.2f tps", mTagSpeed]];
    [self.tableView reloadData];
}
- (void)viewWillAppear:(BOOL)animated {
    [super viewWillAppear:animated];
    [AppDelegate showGlobalProgressHUDWithTitle:@"Loading"];
    p252bManager.delegate = self;
    [self setInventorySession:p252bManager.asReaderP252B.inventorySession];
    [self setSessionFlag:p252bManager.asReaderP252B.sessionFlag];
    if ([self.navigationController respondsToSelector:@selector(interactivePopGestureRecognizer)]) {
        self.navigationController.interactivePopGestureRecognizer.enabled = NO;
    }
}

- (void)viewDidAppear:(BOOL)animated{
    mTotalTagCount = 0;
    mTotalTagCounts = 0;
    for (AsTagItem * item in aryTags) {
        mTotalTagCounts  =  mTotalTagCounts +  item.count;
    }
    mTagSpeed = 0.0;
    isJapan = [[p252bManager.asReaderP252B regionName] hasPrefix:@"Japan"];
    currentTagCount = 0;
    
    [self.speedLabel setText:@"0 tps"];
    [self.reportRssiSwitch setOn:NO];
    isCanScan = YES;

    [self initPowerGainPickerData];
    
    int nPower = p252bManager.asReaderP252B.powerGain;
    NSInteger operationTime = p252bManager.asReaderP252B.operationTime;
    
    mLastTime = 0;
    [self setPowerGain:(int)(nPower / 10)];

    p252bManager.asReaderP252B.continuousMode = [systemSetting getContinuousMode];
    [self.continuousModeSwitch setOn:p252bManager.asReaderP252B.continuousMode];
    p252bManager.asReaderP252B.rssiMode = [systemSetting getRssiMode];
    [self.reportRssiSwitch setOn:p252bManager.asReaderP252B.rssiMode];
    [self.displayPcSwitch setOn:[systemSetting getDisPlayPC]];
    [self setOperationTime:[NSString stringWithFormat:@"%ld", (long)operationTime]];
    [self setInventorySession:p252bManager.asReaderP252B.inventorySession];
    [self setSessionFlag:p252bManager.asReaderP252B.sessionFlag];
    [p252bManager.asReaderP252B setScanMode:AsReaderP252BRFIDScanMode];

    [self.pickerView reloadAllComponents];
    [self updateUI];

    [AppDelegate dismissGlobalHUD];
}

- (void)viewWillDisappear:(BOOL)animated {
    [super viewWillDisappear:animated];
    p252bManager.delegate = nil;
    [p252bManager.asReaderP252B stopSync];
    [self enableControlByActionState];
}

- (void)didReceiveMemoryWarning {
    [super didReceiveMemoryWarning];
}

#pragma mark - CSV Export
- (void)csvExport {
    dispatch_async(dispatch_get_main_queue(), ^{
        if (self->aryTags && self->aryTags.count != 0) {
            NSString *timestr = [self currentData];
            NSString *fileName = [NSString stringWithFormat:@"RFID_%@.csv", timestr];
                //创建csv文件
            [CSVData createCSVWithFileName:fileName export:rfidExportType];
            [CSVData exportRfidCSV:fileName withArray:self->aryTags export:rfidExportType isEpc:self.displayPcSwitch.isOn ];
        }
    });
}

- (void)excelExport {
    dispatch_async(dispatch_get_main_queue(), ^{
        if (self->aryTags && self->aryTags.count != 0) {
            NSString *timestr = [self currentData];
            NSString *fileName = [NSString stringWithFormat:@"RFID_%@.xlsx", timestr];
                //创建excel文件
            [ExcelData createExcelWithFileName:fileName export:rfidExportType];
            [ExcelData exportRfidExcel:fileName withArray:self->aryTags export:rfidExportType isEpc:self.displayPcSwitch.isOn];
        }
    });
}

- (NSString *)currentData {
    NSDate *today = [NSDate date];
    NSDateFormatter *df = [[NSDateFormatter alloc] init];
    [df setDateFormat:@"yyyyMMddHHmmss"];
    NSString *str = [df stringFromDate:today];
    return str;
}

#pragma mark - Navigation
- (void)prepareForSegue:(UIStoryboardSegue *)segue sender:(id)sender {
    if ([[segue identifier] isEqualToString:@"ReadMemory"]) {
        ReadMemoryViewController *destinationController = [segue destinationViewController];
        
        NSIndexPath *indexPath = (NSIndexPath *)sender;
        destinationController.maskValue = ((AsTagItem *)[aryTags objectAtIndex:indexPath.row]).tag;
    } else if ([[segue identifier] isEqualToString:@"WriteMemory"]) {
        WriteMemoryViewController *destinationController = [segue destinationViewController];
        
        NSIndexPath *indexPath = (NSIndexPath *)sender;
        destinationController.maskValue = ((AsTagItem *)[aryTags objectAtIndex:indexPath.row]).tag;
    } else if ([[segue identifier] isEqualToString:@"LockMemory"]) {
        TagAccessViewController *destinationController = [segue destinationViewController];
        
        NSIndexPath *indexPath = (NSIndexPath *)sender;
        destinationController.maskValue = ((AsTagItem *)[aryTags objectAtIndex:indexPath.row]).tag;
    } else if ([[segue identifier] isEqualToString:csvInvenShow]) {
        CSVExportTableViewController *exportVC = segue.destinationViewController;
        if (1 == _csvType) {
            exportVC.typeStr = @"csvInven";
        } else {
            exportVC.typeStr = @"excelInven";
        }
    }
}

#pragma mark - Storyboard Segue
-(IBAction)prepareForUnwind:(UIStoryboardSegue *)segue {
}

#pragma mark - UITableViewDataSource
- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView {
    return 1;
}

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section {
    return aryTags.count;
}

#pragma mark - UITableViewDelegate
- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath {
    InventoryTableViewCell *cell = [tableView dequeueReusableCellWithIdentifier:@"InventoryCell" forIndexPath:indexPath];
    
    AsTagItem *item = nil;
    item = [aryTags objectAtIndex:indexPath.row];
    if (item != nil) {
        if (self.displayPcSwitch.isOn) {
            cell.tagLabel.text = item.tag;
        } else {
            cell.tagLabel.text = [item.tag substringFromIndex:TAG_PC_LENGTH];
        }
        cell.countLabel.text = [NSString stringWithFormat:@"%d", item.count];
        
        if (self.reportRssiSwitch.isOn) {
            cell.rssiLabel.text = [NSString stringWithFormat:@"%.1f dB", item.rssi];
            cell.phaseLabel.text = [NSString stringWithFormat:@"%d˚", (int)item.phase];
            cell.frequency.text =  [NSString stringWithFormat:@"%.3fMHz ",item.frequency];
        } else {
            cell.rssiLabel.text = @"";
            cell.phaseLabel.text = @"";
            cell.frequency.text = @"";
        }
    }
    return cell;
}

- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath {
    
    if ([p252bManager.asReaderP252B getAction] != AsReaderP252BCommandStop)
        {
        return;
        }
    
    UIAlertController *alert = [UIAlertController alertControllerWithTitle:@"Additional Actions"
                                                                   message:@"You can choose one of the following actions for this Tag"
                                                            preferredStyle:UIAlertControllerStyleAlert];
    
    [alert addAction:[UIAlertAction actionWithTitle:@"Read Memory"
                                              style:UIAlertActionStyleDefault
                                            handler:^(UIAlertAction * action) {
        [self nowViewWillDisappear];
        [self performSegueWithIdentifier:@"ReadMemory" sender:indexPath];
    }]];
    
    [alert addAction:[UIAlertAction actionWithTitle:@"Write Memory"
                                              style:UIAlertActionStyleDefault
                                            handler:^(UIAlertAction * action) {
        [self nowViewWillDisappear];
        [self performSegueWithIdentifier:@"WriteMemory" sender:indexPath];
    }]];
    
    [alert addAction:[UIAlertAction actionWithTitle:@"Lock Memory"
                                              style:UIAlertActionStyleDefault
                                            handler:^(UIAlertAction * action) {
        [self nowViewWillDisappear];
        [self performSegueWithIdentifier:@"LockMemory" sender:indexPath];
    }]];
    [alert addAction:[UIAlertAction actionWithTitle:@"OpenLED"
                                              style:UIAlertActionStyleDefault
                                            handler:^(UIAlertAction * action) {
        [AppDelegate showGlobalProgressHUDWithTitle:@""];
        NSString * maskValue = ((AsTagItem *)[self->aryTags objectAtIndex:indexPath.row]).tag;
        [self->appDelegate initMask:maskValue];
        __block int count = 6;
        NSOperationQueue * readOperationQueue = [[NSOperationQueue alloc] init];
        [readOperationQueue addOperationWithBlock:^{
            while (count) {
                [NSThread sleepForTimeInterval:1];
                [[NSOperationQueue mainQueue] addOperationWithBlock:^{
                    if([self->p252bManager.asReaderP252B getAction] != AsReaderP252BCommandStop){
                        [self->p252bManager.asReaderP252B stop];
                    }
                    [self->p252bManager.asReaderP252B readMemory:AsReaderP252BBank_User offset:112 length:1];
                }];
                count--;
            }
            [[NSOperationQueue mainQueue] addOperationWithBlock:^{
                [self->p252bManager.asReaderP252B stop];
                [self->appDelegate exitMask];
                [self->_tableView reloadData];
                [AppDelegate dismissGlobalHUD];
            }];
        }];
    }]];
    [alert addAction:[UIAlertAction actionWithTitle:@"Cancel"
                                              style:UIAlertActionStyleCancel
                                            handler:^(UIAlertAction *action) {

    }]];
    
    [self presentViewController:alert animated:YES completion:nil];
}

#pragma mark - UIPickerViewDataSource
- (NSInteger)numberOfComponentsInPickerView:(UIPickerView *)pickerView
{
    return 1;
}
- (NSInteger)pickerView:(UIPickerView *)pickerView numberOfRowsInComponent:(NSInteger)component
{
    if (pickerView.tag == PICKER_VIEW_POWER_GAIN) {
        return [self.powerGainPickerData count];
    } else if (pickerView.tag == PICKER_VIEW_INVENTORY_SESSION) {
        return self.inventorySessionPickerData.count;
    } else if (pickerView.tag == PICKER_VIEW_SESSION_FLAG) {
        return self.sessionFlagPickerData.count;
    }
    return 0;
}

#pragma mark - UIPickerViewDelegate
- (NSString *)pickerView:(UIPickerView *)pickerView titleForRow:(NSInteger)row forComponent:(NSInteger)component
{
    if (pickerView.tag == PICKER_VIEW_POWER_GAIN) {
        return [self.powerGainPickerData objectAtIndex:row];
    } else if (pickerView.tag == PICKER_VIEW_INVENTORY_SESSION) {
        return [self.inventorySessionPickerData objectAtIndex:row];
    } else if (pickerView.tag == PICKER_VIEW_SESSION_FLAG) {
        return [self.sessionFlagPickerData objectAtIndex:row];
    }
    return @"None";
}

#pragma mark - UITextViewDelegate
- (BOOL)textFieldShouldBeginEditing:(UITextField *)textField
{
    return false;
}

#pragma mark - UIAlertViewDelegate
- (void)alertView:(UIAlertView *)alertView clickedButtonAtIndex:(NSInteger)buttonIndex
{
    if (alertView.tag == ALERT_VIEW_OPERATION_TIME) {
        if (buttonIndex == 1) {
            NSString * value = [alertView textFieldAtIndex:0].text;
            if(value.length == 0){
                value = @"0";
            }
            if ([value intValue] > 1800) {
                value = @"1800";
            }
            [self setOperationTime:value];
            p252bManager.asReaderP252B.operationTime = [value intValue];
            [systemSetting setOperationTime:[value intValue]];
        }
    }
}

#pragma mark - IBActions
- (IBAction)powerGainBtnTapped:(id)sender {
    self.pickerView.tag = PICKER_VIEW_POWER_GAIN;
    [self.pickerView reloadAllComponents];
    if (![self.powerGainTextField.text isEqualToString:@""]) {
        [self.pickerView selectRow:[self.powerGainPickerData indexOfObject:self.powerGainTextField.text] inComponent:0 animated:NO];
    } else {
        [self.pickerView selectRow:0 inComponent:0 animated:NO];
    }
    self.pickerView.hidden = NO;
    self.powerGainPickerTopView.hidden = NO;
}

- (IBAction)displayPcSwitchTapped:(id)sender {
    [systemSetting setDisPlayPC:[self.displayPcSwitch isOn]];
    [self.tableView reloadData];
}

- (IBAction)continuousModeSwitchTapped:(id)sender {
    p252bManager.asReaderP252B.continuousMode = [self.continuousModeSwitch isOn];
    [systemSetting setContinuousMode:[self.continuousModeSwitch isOn]];
}

- (IBAction)reportRssiSwitchTapped:(id)sender {
    p252bManager.asReaderP252B.rssiMode = [self.reportRssiSwitch isOn];
    [systemSetting setRssiMode:[self.reportRssiSwitch isOn]];
    [aryTags removeAllObjects];
    [mapTags removeAllObjects];
    [appDelegate.insertOperation cancelAllOperations];
    [[databaseManeger getDatebaseQueue] inTransaction:^(FMDatabase *db, BOOL *rollback) {
        if (![databaseManeger deleteAllAsTagItemsFmdb:db]) {
            *rollback = YES;
        }
    }];
    currentTagCount = 0;
    [self.tagCountLabel setText:[NSString stringWithFormat:@"%d", (int)aryTags.count]];
    [self.totalTagCountLabel setText:[NSString stringWithFormat:@"%d", (int)aryTags.count]];
    [self.speedLabel setText:@"0 tps"];
    mTagSpeed = 0.0;
    mLastTime = 0;
    mTotalTagCount = 0;
    mTotalTagCounts = 0;
    [tagArray removeAllObjects];
    [self.tableView reloadData];
}

- (IBAction)inventoryBtnTapped:(id)sender {
    if ([p252bManager.asReaderP252B getAction] == AsReaderP252BCommandStop) {
        [p252bManager.asReaderP252B inventory];
    } else {
        [p252bManager.asReaderP252B stop];
    }
}

- (IBAction)clearBtnTapped:(id)sender {
    [aryTags removeAllObjects];
    [mapTags removeAllObjects];
    [self.tagCountLabel setText:[NSString stringWithFormat:@"%d", (int)aryTags.count]];
    [self.totalTagCountLabel setText:[NSString stringWithFormat:@"%d", (int)aryTags.count]];
    [self.speedLabel setText:@"0 tps"];
    mTagSpeed = 0.0;
    mLastTime = 0;
    mTotalTagCount = 0;
    currentTagCount = 0;
    mTotalTagCounts = 0;
    [tagArray removeAllObjects];
    [appDelegate.insertOperation cancelAllOperations];
    [[databaseManeger getDatebaseQueue] inTransaction:^(FMDatabase *db, BOOL *rollback) {
        if (![databaseManeger deleteAllAsTagItemsFmdb:db]) {
            *rollback = YES;
        }
    }];
    [self.tableView reloadData];
    
}

- (IBAction)maskBtnTapped:(id)sender {
    [self nowViewWillDisappear];
    [self performSegueWithIdentifier:@"EpcMask" sender:self];
}

- (IBAction)pickerCloseBtnTapped:(id)sender {
    [self hidePickerView];
}

- (IBAction)powerGainPickerOkBtnTapped:(id)sender {
    if (self.pickerView.tag == PICKER_VIEW_POWER_GAIN) {
        NSInteger idx = [self.pickerView selectedRowInComponent:0];
        idx += (p252bManager.asReaderP252B.powerGainRange.min / 10);
        [self setPowerGain:idx];
        p252bManager.asReaderP252B.powerGain = (int)idx * 10;
        [systemSetting setPowerGain:(int)idx * 10];
    }else if(self.pickerView.tag == PICKER_VIEW_INVENTORY_SESSION){
        [self setInventorySession:(int)[self.pickerView selectedRowInComponent:0]];
        [p252bManager.asReaderP252B setInventorySession:self.inventorySession];
        [systemSetting setInventorySession:self.inventorySession];
    }else if(self.pickerView.tag == PICKER_VIEW_SESSION_FLAG){
        [self setSessionFlag:(int)[self.pickerView selectedRowInComponent:0]];
        [p252bManager.asReaderP252B setSessionFlag:self.sessionFlag];
        [systemSetting setSessionFlag:self.sessionFlag];
    }
    [self hidePickerView];
    
}

- (IBAction)operationTimeBtnTapped:(id)sender {
    UIAlertView *alert = [[UIAlertView alloc] initWithTitle:@"Operation Time:" message:@"Please enter operation time.\n(0~1800s)" delegate:self cancelButtonTitle:@"Cancel" otherButtonTitles:nil];
    
    alert.alertViewStyle = UIAlertViewStylePlainTextInput;
    [alert textFieldAtIndex:0].keyboardType = UIKeyboardTypeNumberPad;
    alert.tag = ALERT_VIEW_OPERATION_TIME;
    
    [alert addButtonWithTitle:@"OK"];
    [alert show];
}

- (IBAction)csvOutputBtnTapped:(id)sender {
    [self presentViewController:self.actionAlertVC animated:NO completion:nil];
}

- (IBAction)backBtnTapped:(id)sender {
    [self nowViewWillDisappear];
    [self.navigationController popViewControllerAnimated:YES];
}


#pragma mark - Getters
- (NSArray *)inventorySessionPickerData {
    return DATA_INVENTORY_SESSION;
}
- (NSArray *)sessionFlagPickerData {
    return DATA_SESSION_FLAG;
}

#pragma mark - Init
- (void)initPowerGainPickerData {
    if (_powerGainPickerData == nil) {
        NSMutableArray *tmpArray = [[NSMutableArray alloc] init];
        int min = p252bManager.asReaderP252B.powerGainRange.min;
        int max = p252bManager.asReaderP252B.powerGainRange.max;
        
        for (int i = min; i <= max; i += 10) {
            [tmpArray addObject:[NSString stringWithFormat:@"%i.0 dB", i / 10]];
        }
        _powerGainPickerData = tmpArray;
    }
}

#pragma mark - Custom
- (void) hidePickerView {
    self.powerGainPickerTopView.hidden = YES;
    self.pickerView.hidden = YES;
}

- (void)setInventorySession:(AsReaderP252BSessionType)inventorySession {
    self.inventorySessionTextField.text = [self.inventorySessionPickerData objectAtIndex:inventorySession];
    _inventorySession = inventorySession;
}

- (void)setPowerGain:(NSInteger)power {
    NSString *value = [NSString stringWithFormat:@"%li.0 dB", (long)power];
    NSInteger index = [self.powerGainPickerData indexOfObject:value];
    if ([self.powerGainPickerData count] >index) {
        self.powerGainTextField.text = [self.powerGainPickerData objectAtIndex:index];
    }
}
- (void)setSessionFlag:(AsReaderP252BSessionFlag)sessionFlag {
    self.sessionFlagTextField.text = [self.sessionFlagPickerData objectAtIndex:sessionFlag];
    _sessionFlag = sessionFlag;
}

- (void)setOperationTime:(NSString *)operationTime {
    self.operationTimeTextField.text = operationTime;
}
- (void)changStatus{
    isCanScan = YES;
}
#pragma mark - Reader Event Handler Methods
- (BOOL)onAsReaderP252BTriggerKeyEvent:(BOOL)status{
    if (status) {
        [self inventoryBtnTapped:nil];
    }
    return NO;
}
- (BOOL)onAsReaderP252BModeKeyEvent:(BOOL)status{
    return NO;
}
- (void)updateAsReaderP252BDeviceState:(AsReaderP252BResultCode)error {
    if (error != AsReaderP252BResultNoError) {
        UIAlertView *alert = [[UIAlertView alloc]initWithTitle:@"Error" message:@"Failed to get response. Try again." delegate:nil cancelButtonTitle:@"OK" otherButtonTitles:nil];
        [alert show];
        [self.navigationController popViewControllerAnimated:YES];
    }
}

- (void)readAsReaderP252BTag:(NSString *)tag rssi:(float)rssi phase:(float)phase frequency:(float)frequency {
    __block float rssiBlock = rssi;
    __block float phaseBlock = phase;
    __block float frequencyBlock = frequency;
    dispatch_queue_t queue =  dispatch_queue_create("queue",DISPATCH_QUEUE_SERIAL);
    dispatch_async(queue, ^{
        long time = CFAbsoluteTimeGetCurrent();
        double interval = 0.0;
        
        self->mTotalTagCount++;
        self->mTotalTagCounts++;
        if (mLastTime == 0) {
            mTagSpeed = 0.0;
            mLastTime = time;
        } else {
            interval = (double) (time - mLastTime);
            if (interval != 0)
                mTagSpeed = (double) (self->mTotalTagCount - self->currentTagCount)/ interval;
        }
        
        if ([self->systemSetting getRssiMode] == NO) {
            rssiBlock = 0.0;
            phaseBlock = 0.0;
        }
        
        AsTagItem *item = nil;
        @synchronized (self) {
            @try {
                if ((item = (AsTagItem *)[self->mapTags objectForKey:tag]) != nil) {
                    item.count++;
                    [item updateRssi:rssiBlock withPhase:phaseBlock frequency:frequencyBlock];
                } else {
                    item = [AsTagItem itemWith:tag rssi:rssiBlock phase:phaseBlock frequency:frequencyBlock];
                    [self->aryTags addObject:item];
                    [self->mapTags setObject:item forKey:tag];
                }
                
                NSString * sql = [NSString stringWithFormat:@"REPLACE INTO AsTagItem \
                                  (tag\
                                  ,count\
                                  ,rssi\
                                  ,phase\
                                  ,frequency\
                                  ,datetime) \
                                  VALUES ('%@','%@','%@','%@','%@','%@')",
                                  item.tag,
                                  [NSString stringWithFormat:@"%d",item.count],
                                  [NSString stringWithFormat:@"%f",item.rssi],
                                  [NSString stringWithFormat:@"%f",item.phase],
                                  [NSString stringWithFormat:@"%f",item.frequency],
                                  item.dateTime];
                [self->sqlDic setValue:sql forKey:item.tag];
                if ([[self->sqlDic allValues] count] > 20) {
                    [self replaceDB];
                }
            } @catch (NSException *exception) {

            } @finally {

            }
            
            dispatch_async(dispatch_get_main_queue(), ^{
                [self.tagCountLabel setText:[NSString stringWithFormat:@"%d", (int)self->aryTags.count]];
                [self.totalTagCountLabel setText:[NSString stringWithFormat:@"%ld", self->mTotalTagCounts]];
                [self.speedLabel setText:[NSString stringWithFormat:@"%0.2f tps", mTagSpeed]];
                [self.tableView reloadData];
            });
        }
    });
}
- (void)changedAsReaderP252BActionState:(AsReaderP252BCommandType)action resultCode:(NSInteger)resultCode {
    if (action == AsReaderP252BCommandStop){
        [self replaceDB];
        [self.inventoryBtn setTitle:@"Inventory" forState:UIControlStateNormal];
        mLastTime = 0;
        currentTagCount = mTotalTagCount;
        [[self toDisableElems]enumerateObjectsUsingBlock:^(id  _Nonnull obj, NSUInteger idx, BOOL * _Nonnull stop) {
            UIControl *control = obj;
            [control setEnabled:YES];
        }];
        self.csvOutputBtn.enabled = YES;
        self.tableView.userInteractionEnabled = YES;
        if (resultCode == AsReaderP252BResultNoError) {
            isCanScan = YES;
            [NSObject cancelPreviousPerformRequestsWithTarget:self selector:@selector(changStatus) object:nil];
        }else{
            isCanScan = NO;
            [NSObject cancelPreviousPerformRequestsWithTarget:self selector:@selector(changStatus) object:nil];
            [self performSelector:@selector(changStatus) withObject:nil afterDelay:1.0];
        }
    }else{
        if (resultCode == AsReaderP252BResultNoError)
            {
            [self.inventoryBtn setTitle:@"Stop" forState:UIControlStateNormal];
            [[self toDisableElems]enumerateObjectsUsingBlock:^(id  _Nonnull obj, NSUInteger idx, BOOL * _Nonnull stop) {
                UIControl *control = obj;
                [control setEnabled:NO];
            }];
            self.csvOutputBtn.enabled = NO;
            self.tableView.userInteractionEnabled = NO;

            }
    }

}
- (void)accessAsReaderP252BResult:(AsReaderP252BResultCode)error actionState:(AsReaderP252BCommandType)action epc:(NSString *)epc data:(NSString *)data rssi:(float)rssi phase:(float)phase frequency:(float)frequency{

}
- (void)enableControlByActionState {
    if ([p252bManager.asReaderP252B getAction] == AsReaderP252BCommandStop){
        [self replaceDB];
        [self.inventoryBtn setTitle:@"Inventory" forState:UIControlStateNormal];
        mLastTime = 0;
        currentTagCount = mTotalTagCount;
        [[self toDisableElems]enumerateObjectsUsingBlock:^(id  _Nonnull obj, NSUInteger idx, BOOL * _Nonnull stop) {
            UIControl *control = obj;
            [control setEnabled:YES];
        }];
        self.csvOutputBtn.enabled = YES;
        self.tableView.userInteractionEnabled = YES;
    }
    
}

- (IBAction)onClickOptionButton:(id)sender {
    CGRect frame  = self.tableView.frame;
    CGRect frameS = self.scrollView.frame;
    if (frame.size.height > frameS.size.height ) {
        frame.size.height =  frame.size.height  - frameS.size.height;
        self.scrollView.hidden = NO;
    }else{
        frame.size.height =  frame.size.height  + frameS.size.height;
        self.scrollView.hidden = YES;
    }
    self.tableView.frame = frame;
}
- (IBAction)onClickInventorySessionButton:(id)sender {
    self.pickerView.tag = PICKER_VIEW_INVENTORY_SESSION;
    [self.pickerView reloadAllComponents];
    
    [self.pickerView selectRow:self.inventorySession inComponent:0 animated:NO];
    
    self.pickerView.hidden = NO;
    self.powerGainPickerTopView.hidden = NO;
}
- (IBAction)onClickSessionFlagButton:(id)sender {
    self.pickerView.tag = PICKER_VIEW_SESSION_FLAG;
    [self.pickerView reloadAllComponents];
    
    [self.pickerView selectRow:self.sessionFlag inComponent:0 animated:NO];
    
    self.pickerView.hidden = NO;
    self.powerGainPickerTopView.hidden = NO;
}

-(void)whenAsReaderP252BIsConnected:(BOOL)isConnected{
    if (isConnected) {
        dispatch_async(dispatch_get_main_queue(), ^{
            [AppDelegate showGlobalProgressHUDWithTitle:@"Loading..."];
            [CATransaction commit];
            [AppDelegate dismissGlobalHUD];
            [CATransaction commit];
        });
    }else{
        dispatch_async(dispatch_get_main_queue(), ^{
            [self nowViewWillDisappear];
            [self.navigationController popToRootViewControllerAnimated:YES];
        });
    }
}
- (void)nowViewWillDisappear{
    [self replaceDB];
    if (appDelegate.insertOperation.operationCount > 0) {
        [AppDelegate dismissGlobalHUD];
        [AppDelegate showGlobalProgressHUDWithTitle:@"Data Insert..."];
        [CATransaction commit];
        while (true) {
            [NSThread sleepForTimeInterval:0.5];
            if (appDelegate.insertOperation.operationCount==0) {
                break;
            }
        }
        [AppDelegate dismissGlobalHUD];
        [CATransaction commit];
    }else{
        return;
    }
}

- (void)replaceDB{
    if ([[sqlDic allKeys] count] == 0) {
        return;
    }
    NSDictionary * dic = [sqlDic copy];
    [sqlDic removeAllObjects];
    [appDelegate.insertOperation addOperationWithBlock:^{
        [[self->databaseManeger getDatebaseQueue] inTransaction:^(FMDatabase *db, BOOL *rollback) {
            for (NSString * str in  [dic allValues]) {
                if (![self->databaseManeger updataOrAddSql:str fmdb:db]) {
                    *rollback = YES;
                }
            }
        }];
    }];
}

@end
