#import "MainViewController.h"
#import "MainButton.h"
#import "VersionHandler.h"
#import "SystemSetting.h"
#import "AsReaderP252BManager.h"
#define BATTERY_NONE            0
#define BATTERY_QUARTER         1
#define BATTERY_HALF            2
#define BATTERY_THIRD_QUARTER   3
#define BATTERY_FULL            4

#define CONNECTION_OFF          0
#define CONNECTION_ON           1

@interface MainViewController ()<AsReaderP252BManagerDelegate,UITextFieldDelegate,VersionHandlerDelegate>
{
    NSString *updateUrl;
    AsReaderP252BManager *p252bManager;
    NSTimer *batteryTimer;
}
@property (weak, nonatomic) IBOutlet UILabel *asRingPlusVersion;
@property (weak, nonatomic) IBOutlet MainButton *btnInventory;
@property (strong, nonatomic) IBOutletCollection(MainButton) NSArray *buttonCollection;
@property (weak, nonatomic) IBOutlet UIImageView *batteryImage;
@property (weak, nonatomic) IBOutlet UIImageView *logoImage;
@property (weak, nonatomic) IBOutlet UILabel *labelDemoVer;
@property (weak, nonatomic) IBOutlet UILabel *labelFirmwareVer;
@property (weak, nonatomic) IBOutlet UILabel *regionLabel;
@property (weak, nonatomic) IBOutlet UILabel *snLabel;
@property (weak, nonatomic) IBOutlet UILabel *ufVersion;

@end

@implementation MainViewController

- (void)viewDidLoad {
    [super viewDidLoad];
    if (@available(iOS 15.0, *)){
        UINavigationBarAppearance *appearance = [[UINavigationBarAppearance alloc] init];
        appearance.backgroundEffect = [UIBlurEffect effectWithStyle:UIBlurEffectStyleRegular];
        appearance.backgroundColor = [UIColor whiteColor];
        self.navigationController.navigationBar.scrollEdgeAppearance = appearance;
    }
    p252bManager = [AsReaderP252BManager sharedAsReaderP252BManager];
    self.labelDemoVer.text = [self getAppVersion];
    self.regionLabel.text = @"";
    self.snLabel.text = @"";
    self.ufVersion.text = @"";    
//    dispatch_async(dispatch_get_global_queue(DISPATCH_QUEUE_PRIORITY_HIGH, 0), ^{
//        [VersionHandler sharedVersionHandler].versionHandlerDelegate=self;
//        [[VersionHandler sharedVersionHandler] getServeVersionWithUrl:@"https://www.asteriskweb.jp/mrxapp/AsReaderLR/manifest.plist"];
//    });
}
- (NSString *)getAppVersion {
    NSDictionary *infoDictionary = [[NSBundle mainBundle] infoDictionary];
    NSString *majorVersion = [infoDictionary objectForKey:@"CFBundleShortVersionString"];
    NSString *minorVersion = [infoDictionary objectForKey:@"CFBundleVersion"];
    return [NSString stringWithFormat:@"Version %@ Build %@", majorVersion, minorVersion];
}

- (void)viewWillAppear:(BOOL)animated {
    [super viewWillAppear:animated];
    p252bManager.delegate = self;
}
- (void)viewDidAppear:(BOOL)animated{
    if (p252bManager.asReaderP252B.isConnected) {
        if (p252bManager.settingRFIDType == AsReaderP252BSettingRFIDType_Set) {
            [AppDelegate dismissGlobalHUD];
            [AppDelegate showGlobalProgressHUDWithTitle:@"Loading..."];
            self.labelFirmwareVer.text = p252bManager.asReaderP252B.firmwareVersion;
            self.asRingPlusVersion.text =[p252bManager.asReaderP252B hardwareVersion];
            self.regionLabel.text = [p252bManager.asReaderP252B regionName];
            self.snLabel.text = [p252bManager.asReaderP252B serialNumber];
            self.ufVersion.text = [p252bManager.asReaderP252B rFModuleVersion];
            [AppDelegate dismissGlobalHUD];
            [AppDelegate showGlobalProgressHUDWithTitle:@"RFID Setting..."];
            [[SystemSetting sharedSystemSetting] RFIDSetting];
            [AppDelegate dismissGlobalHUD];
            [self enableAllButtons];
            [self startTiemr];
            [AppDelegate dismissGlobalHUD];
            p252bManager.settingRFIDType = AsReaderP252BSettingRFIDType_Seted;
        } else if (p252bManager.settingRFIDType == AsReaderP252BSettingRFIDType_Seted) {
            [self startTiemr];
        }
    }else{
        [p252bManager startDiscovered];
        [self disableAllUIs];
    }
}

- (void)viewWillDisappear:(BOOL)animated{
    [super viewWillDisappear:animated];
    p252bManager.delegate = nil;
    if (batteryTimer) {
        [batteryTimer invalidate];
        batteryTimer = nil;
    }
}
- (void)didReceiveMemoryWarning {
    [super didReceiveMemoryWarning];
}


#pragma mark - Navigation
- (void)prepareForSegue:(UIStoryboardSegue *)segue sender:(id)sender {
	
}

#pragma mark - Storyboard Segue
-(IBAction)prepareForUnwind:(UIStoryboardSegue *)segue {
}

#pragma mark - Connect/Disconnect Peripheral Methods
- (void)disconnectDevice {
    [self disableAllUIs];
}

#pragma mark - Reader Event Handler Methods
-(void)whenAsReaderP252BIsConnected:(BOOL)isConnected{
    if (isConnected) {
        [AppDelegate dismissGlobalHUD];
        [AppDelegate showGlobalProgressHUDWithTitle:@"AsReaderP252B Initializing..."];
    }else{
        [self disableAllUIs];
        [AppDelegate dismissGlobalHUD];
    }
    [self viewDidAppear:YES];
}
- (BOOL)onAsReaderP252BTriggerKeyEvent:(BOOL)status{
    return NO;
}

- (void)readAsReaderP252BTag:(NSString *)tag rssi:(float)rssi phase:(float)phase {
    
}

- (void)changedAsReaderP252BActionState:(AsReaderP252BCommandType)action resultCode:(NSInteger)resultCode {
    [self enableControlByActionState];
}

- (void)enableControlByActionState {
    
}

#pragma mark - UIAlertViewDelegate
- (void)alertView:(UIAlertView *)alertView clickedButtonAtIndex:(NSInteger)buttonIndex {
    switch ([alertView tag]) {
        case kBluetoothAlertView:
            break;
        case kDeviceAlertView:
            if (p252bManager.asReaderP252B.isConnected) {
            } else {
                [self disconnectDevice];
            }
            break;
        case VersionAlertView:
            if (buttonIndex==0) {
                
            }else{
                [[UIApplication sharedApplication]openURL:[NSURL URLWithString:updateUrl] options:@{} completionHandler:^(BOOL success) {
                    
                }];
            }
            break;
        case VersionForceAlertView:
            if (buttonIndex==0) {
                [[UIApplication sharedApplication]openURL:[NSURL URLWithString:updateUrl] options:@{} completionHandler:^(BOOL success) {
                    
                }];
            }else{
                
            }
            break;
        default:
            break;
    }
}

#pragma mark - Custom
- (void)enableAllButtons {
    for (MainButton *btn in self.buttonCollection) {
        [btn enableButton];
    }
    [self changeLogoImage:CONNECTION_ON];
}

- (void)disableAllUIs {
    self.asRingPlusVersion.text = @"";
    self.labelFirmwareVer.text = @"";
    self.regionLabel.text = @"";
    self.snLabel.text = @"";
    self.ufVersion.text = @"";
    for (MainButton *btn in self.buttonCollection) {
        [btn disableButton];
    }
    [self changeBatteryImage:BATTERY_NONE];
    [self changeLogoImage:CONNECTION_OFF];
}

- (void)disableRFIDButtons {
    for (MainButton *btn in self.buttonCollection) {
        [btn disableButton];
    }
}
- (void)changeBatteryImage:(NSUInteger)batteryStatus {
    if (batteryStatus == BATTERY_NONE) {
        [self.batteryImage setImage:[UIImage imageNamed:@"battery0"]];
    } else if (batteryStatus == BATTERY_QUARTER) {
        [self.batteryImage setImage:[UIImage imageNamed:@"battery1"]];
    } else if (batteryStatus == BATTERY_HALF) {
        [self.batteryImage setImage:[UIImage imageNamed:@"battery2"]];
    } else if (batteryStatus == BATTERY_THIRD_QUARTER) {
        [self.batteryImage setImage:[UIImage imageNamed:@"battery3"]];
    } else if (batteryStatus == BATTERY_FULL) {
        [self.batteryImage setImage:[UIImage imageNamed:@"battery4"]];
    }
}

- (void)changeLogoImage:(NSUInteger)connectionStatus {
    if (connectionStatus == CONNECTION_OFF) {
        [self.logoImage setImage:[UIImage imageNamed:@"asreader_logo_off"]];
    } else if (connectionStatus == CONNECTION_ON) {
		[self.logoImage setImage:[UIImage imageNamed:@"asreader_logo_on"]];
    }
}

- (void)startTiemr{
    NSString *batteryInterval = [[SystemSetting sharedSystemSetting] getBatteryInterval];
    [self gatBattery];
    if (batteryInterval.integerValue == 0) {
        return;
    }
    if (batteryTimer) {
        [batteryTimer invalidate];
        batteryTimer = nil;
    }
    batteryTimer = [NSTimer scheduledTimerWithTimeInterval:batteryInterval.integerValue target:self selector:@selector(gatBattery) userInfo:nil repeats:YES];
    [[NSRunLoop mainRunLoop] addTimer:batteryTimer forMode:NSRunLoopCommonModes];
}
- (void)gatBattery {
    if (p252bManager.asReaderP252B.isConnected) {
        int batteryStatus = p252bManager.asReaderP252B.batteryStatus;
        [self changeBatteryImage:batteryStatus];
    }
}
 - (void)accessAsReaderP252BResult:(AsReaderP252BResultCode)error actionState:(AsReaderP252BCommandType)action epc:(NSString *)epc data:(NSString *)data rssi:(float)rssi phase:(float)phase  frequency:(float)frequency{
    if (error != AsReaderP252BResultNoError) {
        UIAlertView *alert = [[UIAlertView alloc] initWithTitle:@"Error" message:[p252bManager.asReaderP252B getMessageWithAsReaderP252BResultCode:error] delegate:nil cancelButtonTitle:@"OK" otherButtonTitles:nil];
        [alert show];
        return;
    }
}

#pragma mark VersionHandlerDelegate methods
- (void)UpdateUrl:(NSString *)url forceType:(NSString *)forceType{
    dispatch_async(dispatch_get_main_queue(), ^{
        self->updateUrl=url;
        if ([forceType isEqualToString:@"0"]) {
            UIAlertView *alert=[[UIAlertView alloc]initWithTitle:@"Warning" message:@"There is a new version of the Application server, please go to update the latest Application server" delegate:self cancelButtonTitle:@"Cancel" otherButtonTitles:@"Update", nil];
            [alert setTag:VersionAlertView];
            [alert show];
        }else{
            UIAlertView *alert=[[UIAlertView alloc]initWithTitle:@"Warning" message:@"Your Application version is too old, it is necessary to update the latest version of the Application server" delegate:self cancelButtonTitle:@"Update" otherButtonTitles:nil, nil];
            [alert setTag:VersionForceAlertView];
            [alert show];
        }
    });
}

@end
