//
//  SearchViewController.m
//  AsReaderLR
//
//  Created by iOS on 15.6.21.
//  Copyright © 2021 asterisk. All rights reserved.
//

#import "SearchViewController.h"
#import "AsReaderP252BManager.h"
#import "AppDelegate.h"
#import "SystemSetting.h"
#import <ExternalAccessory/ExternalAccessory.h>
@interface SearchViewController ()<UITableViewDelegate,UITableViewDataSource,AsReaderP252BManagerDelegate>

@end

@implementation SearchViewController{
    NSArray *dataArray;
    AsReaderP252BManager *p252bManager;
}

- (void)viewDidLoad {
    [super viewDidLoad];
    p252bManager = [AsReaderP252BManager sharedAsReaderP252BManager];
    p252bManager.delegate = self;
    [p252bManager startDiscovered];
    // Do any additional setup after loading the view.
}
- (void)viewWillAppear:(BOOL)animated{
    [super viewWillAppear:animated];
    [p252bManager setDelegate:self];
    if ([self.navigationController respondsToSelector:@selector(interactivePopGestureRecognizer)]) {
        self.navigationController.interactivePopGestureRecognizer.enabled = NO;
    }
}
- (void)viewWillDisappear:(BOOL)animated{
    [super viewWillDisappear:animated];
    [p252bManager setDelegate:nil];
    [p252bManager stopDiscovered];
}

#pragma mark - AsReaderP252BDelegate
-(void)didDiscoverAsReaderP252BDeviceList:(NSArray<AsReaderP252B *> *)bluetoothDeviceList{
    dataArray = bluetoothDeviceList;
    [_tableVIew reloadData];
}
- (void)whenAsReaderP252BIsConnected:(BOOL)isConnected{
    [self.tableVIew reloadData];
    if (!isConnected) {
        [AppDelegate dismissGlobalHUD];
    }else{
        [AppDelegate dismissGlobalHUD];
        [p252bManager stopDiscovered];
        [[SystemSetting sharedSystemSetting] setIsAutoConnected:YES];
        [self.navigationController popViewControllerAnimated:YES];
    }
}
- (void)didUpdateBluetoothState:(BOOL)stasus{
    NSLog(@"didUpdateBluetoothState %d " ,stasus);
}
#pragma mark - UITableViewDataSource
- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView {
    return 1;
}

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section {
    return dataArray.count;
}

#pragma mark - UITableViewDelegate
- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath {
    NSString *cellIdentifier = @"AsReaderP252B";
    UITableViewCell *cell = [tableView dequeueReusableCellWithIdentifier:cellIdentifier];
    if(!cell){
        cell = [[UITableViewCell alloc] initWithStyle:UITableViewCellStyleSubtitle reuseIdentifier:cellIdentifier];
    }
    AsReaderP252B *asrader = [dataArray objectAtIndex:indexPath.row];
    if (asrader != nil) {
        cell.textLabel.text = asrader.deviceName;
        EAAccessory * accessory = asrader.currentAccessory;
        cell.detailTextLabel.text = [NSString stringWithFormat:@"Serial: %@", accessory.serialNumber];
    }
    cell.selectionStyle =  UITableViewCellSelectionStyleNone;
    if (asrader.isConnected) {
        cell.accessoryType = UITableViewCellAccessoryCheckmark;
    }else{
        cell.accessoryType = UITableViewCellAccessoryNone;
    }
    return cell;
}
- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath{
    AsReaderP252B *asrader = [dataArray objectAtIndex:indexPath.row];
    EAAccessory * accessory = asrader.currentAccessory;
    if(p252bManager.asReaderP252B){
        EAAccessory * accessoryConnected = p252bManager.asReaderP252B.currentAccessory;
        if(![accessoryConnected.serialNumber isEqualToString:accessory.serialNumber]){
            if (p252bManager.asReaderP252B.isConnected){
                [p252bManager.asReaderP252B disconnect];
            }
        }
    }
    if (asrader.isConnected) {
        [AppDelegate dismissGlobalHUD];
        [AppDelegate showGlobalProgressHUDWithTitle:@"DisConnecting..."];
        [asrader disconnect];
        [[SystemSetting sharedSystemSetting] setIsAutoConnected:NO];
    }else{
        [AppDelegate dismissGlobalHUD];
        [AppDelegate showGlobalProgressHUDWithTitle:@"Connecting..."];
        [p252bManager connectWithAsReader:asrader];
        if ([asrader.currentAccessory isKindOfClass:[EAAccessory class]]) {
            EAAccessory * accessory = asrader.currentAccessory;
            [[SystemSetting sharedSystemSetting] setAsReaderP252BName:accessory.serialNumber];
        }
    }
}
@end
